from unv2x import *

# ------ KS: Minor changes to many fil.write lines were made to conform to Impact's FEMBIC format requirements

if __name__=='__main__':
    helpmsg=""" unv2impact: Convert UNV file from Salome to Impact's *.in (FEMBIC) mesh
        usage: unv2impact unvfile prefix
        what it does: read unvfile (IDEAS file), create an internal FEM object structure
                      in memory and write the prefix.in (FEMBIC) file
        All elements in groups named X_* will just be deleted before conversion
        (useful for deleting all additional 1D & 2D elements created by Salome)
        For the moment, all Beam elements are automatically deleted...
        & only linear tri, quads, tetra, hexa & quadratic tri, quads, tetra & hexa
        are supported.
        The Node and Element sets are NOT written out because the formats are 
        very different.  See the comments in the FEMBIC output file for more caveats!
    """
    donemsg="""*** File converted ***"""
    if len(sys.argv)==3:
        unvfile=sys.argv[1]
        prefix=sys.argv[2]
        ls=os.linesep
        # read UNV file in FEM object structure
        UNV=UNVParser(unvfile)
        FEM=UNV.parse()
        # write ELMER UNV file
        # units
        fil=open(prefix + '.in','w')
        fil.write('// Impact input file'+ls)
        fil.write('// Created by unv2impact.py -a modified version of J. Cugnoni code for Unv2Abaqus.py' + ls)
        fil.write('// Node & Element sets cannot be embedded into each line' + ls)
        fil.write('// because the file formats for UNV & FEMBIC are very different !' + ls)
        fil.write('// If found in the UNV file they are listed at the end for completeness.' + ls)
        fil.write('// ' + ls)
        fil.write('// Both Shell and Solid elements are written, with numbers starting from 1' + ls)
        fil.write('// Manually delete whichever group is not needed using a text editor' + ls)
        fil.write('// ' + ls)
        fil.write('// The node definitions end with (Load =  \tConstraint = )' + ls)
        fil.write('// This is to enable easy "Find and Replace" using a text editor' + ls)
        fil.write('// If you want these stripped, use Linux "sed", or a text editor' + ls)
        fil.write('// ' + ls)
        fil.write('// Thickness: T = 1, change to suit your problem, eliminate it for Solid elements!' + ls)
        fil.write('// Number of Integration Points: NIP = 1/5/8, is embedded in each element line!' + ls)
        fil.write('// 2006 -JMB' + ls)
        # node         
        fil.write(ls+'NODES'+ls)
        for node in FEM.nodes:
            fil.write(('%d \tx = %r \ty = %r \tz = %r load = \tconstraint = '+ls) % (node.id,node.coords[0],node.coords[1],node.coords[2]))
        # elements
        #  first delete all elems from X_* groups
        X_Ids=[]        
        for group in FEM.elemsets:
            if group.name.startswith('X_'):
                X_Ids.extend(group.items)
                FEM.elemsets.remove(group)                
        FEM.nelemsets=len(FEM.elemsets)        
        #  split element list by types
        # ------ KS: The types = line was changed to conform to Impact's FEMBIC element names
        types={41:'Shell_C0_3',42:'Shell_C0_6',44:'Shell_BT_4',45:'Shell8R5-S8R5',111:'Solid_Iso_4',115:'Solid_Iso_6',116:'HexaQuad-C3D20',118:'TetraQuad-C3D10'}
        elemdic={41:[],42:[],44:[],45:[],111:[],115:[],116:[],118:[]}
        # ------ JC: here we define a dictionary that defines numbering of the nodes in the element connectivities in FEMBIC format
        #            for example: the node list [0,1,2,3,4,5,6,7] in UNV should be written [0,1,2,3,4,5,6,7] in FEMBIC format for 'Solid_Iso_6'
        #            (PS: remember Python indices start at 0, not 1)
        #            This dictionnary will be used afterwards to convert the list of nodes in each element.
	# ------ KS: The node numbering is the same in UNV & FEMBIC!  (I was mistaken! Oh well, live and learn...)
#        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:range(4),
#                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
#        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:[3,2,1,0],
#                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
#        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:[0,2,1,3],	# Zero mass!
#                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
#        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:[0,1,2,3],	# Seq: 1324 Works but mass is incorrect!
#                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
#        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:[3,2,1,0], # Seq: 4321
#                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
        cntvtnumbering={41:range(3),42:range(6),44:range(4),45:range(8),111:[0,1,2,3],
                        115:[ 0, 1, 2, 3, 4, 5, 6, 7 ],116:range(20),118:range(10)}
        for elem in FEM.elems:
            if elem.type in types.keys():
                elemdic[elem.type].append(elem)
        # write elements
        for typ in elemdic.keys():
            if len(elemdic[typ])>0:                
                fil.write(ls+'Elements of type '+types[typ]+ls)
                nelem = 0
                for elem in elemdic[typ]:
                    if not(elem.id in X_Ids):
                    # ------ KS: nelem = nelem + 1 added to renumber each element set to start at 1
                        nelem = nelem + 1
                        nnodes=elem.nnodes
                        # ------ KS: nelem is used in the fil.write instead of elem.id, with a little output format change
                        # ------ KS:                      fil.write('%8d,' % elem.id )
                        fil.write('%d\tNodes = [' % nelem )
                        count=0
                        # ------ JC: here we use the dictionary to renumber the nodes in the element connectivities from UNV to FEMBIC format
                        #            to do that task in a simple and efficient way, we use an "implicit" loop to define the new list
                        #            example: mylist=[2*x for x in [0,1,2]] will return [0,2,4]
                        lst=[elem.cntvt[i] for i in cntvtnumbering[elem.type]]                        
                        for i in range(nnodes):
                            count=count+1
                            if (count<8)&(i<>nnodes-1):
                                fil.write('%d,' % lst.pop(0))
                            else:
				if (types[typ]=="Solid_Iso_4")|(types[typ]=="Solid_Iso_6"):
                                   fil.write(('%d]\tMaterial = Steel\tNIP = 1'+ls) % lst.pop(0))
                                else:
                                   fil.write(('%d]\tMaterial = Steel\tT = 1.0\tNIP = 5\tFriction = 0.3 Factor = 1000'+ls) % lst.pop(0))
                                count=0
        # group definitions
        for group in FEM.nodesets:
            fil.write(ls+'// Node set name = %s' % group.name)
            count=0            
            lst=group.items                        
            for i in range(group.nitems):
                count=count+1
                if (count<4)&(i<>group.nitems-1):
                    fil.write('%d,\t' % (lst.pop(0)))
                else:
                    fil.write(('%d'+ls) % (lst.pop(0)))
                    count=0
        for group in FEM.elemsets:
            fil.write(ls+'// Element set name = %s' % group.name)
            count=0            
            lst=group.items
            for i in range(group.nitems):
                count=count+1
                if (count<4)&(i<>group.nitems-1):
                    fil.write('%d,\t' % (lst.pop(0)))
                else:
                    fil.write(('%d'+ls) % (lst.pop(0)))
                    count=0
        fil.close()
        print(donemsg)
        print(('%s.unv File converted to Impact format *.in file' + ls) % prefix)
    else:
        print(helpmsg)
